@@ -16,11 +16,6 @@ from oslo_log import versionutils
 from keystone.conf import utils
 
 
-_DEPRECATE_MUTABLE_DOMAIN_IDS = utils.fmt("""
-The option to set domain_id_immutable to false has been deprecated in the M
-release and will be removed in the O release.
-""")
-
 _DEPRECATE_PROXY_SSL = utils.fmt("""
 This option has been deprecated in the N release and will be removed in the P
 release. Use oslo.middleware.http_proxy_to_wsgi configuration instead.
@@ -42,7 +37,7 @@ as it presents a security risk), remove `AdminTokenAuthMiddleware`
 example, in `keystone-paste.ini`).
 """))
 
-public_endpoint = cfg.StrOpt(
+public_endpoint = cfg.URIOpt(
     'public_endpoint',
     help=utils.fmt("""
 The base public endpoint URL for Keystone that is advertised to clients (NOTE:
@@ -54,7 +49,7 @@ the value of the base URL contains a path that keystone does not automatically
 infer (`/prefix/v3`), or if the endpoint should be found on a different host.
 """))
 
-admin_endpoint = cfg.StrOpt(
+admin_endpoint = cfg.URIOpt(
     'admin_endpoint',
     help=utils.fmt("""
 The base admin endpoint URL for Keystone that is advertised to clients (NOTE:
@@ -82,16 +77,16 @@ max_param_size = cfg.IntOpt(
 Limit the sizes of user & project ID/names.
 """))
 
-# we allow tokens to be a bit larger to accommodate PKI
+# NOTE(breton): 255 is the size of the database columns used for ID fields.
+# This size is picked so that the tokens can be indexed in-place as opposed to
+# being entries in a string table. Thus, this is a performance decision.
 max_token_size = cfg.IntOpt(
     'max_token_size',
-    default=8192,
+    default=255,
     help=utils.fmt("""
 Similar to `[DEFAULT] max_param_size`, but provides an exception for token
-values. With PKI / PKIZ tokens, this needs to be set close to 8192 (any higher,
-and other HTTP implementations may break), depending on the size of your
-service catalog and other factors. With Fernet tokens, this can be set as low
-as 255. With UUID tokens, this should be set to 32).
+values. With Fernet tokens, this can be set as low as 255. With UUID tokens,
+this should be set to 32).
 """))
 
 member_role_id = cfg.StrOpt(
@@ -147,21 +142,6 @@ to a reasonable number to prevent operations like listing all users and
 projects from placing an unnecessary load on the system.
 """))
 
-domain_id_immutable = cfg.BoolOpt(
-    'domain_id_immutable',
-    default=True,
-    deprecated_for_removal=True,
-    deprecated_reason=_DEPRECATE_MUTABLE_DOMAIN_IDS,
-    deprecated_since=versionutils.deprecated.MITAKA,
-    help=utils.fmt("""
-Set this to false if you want to enable the ability for user, group and project
-entities to be moved between domains by updating their `domain_id` attribute.
-Allowing such movement is not recommended if the scope of a domain admin is
-being restricted by use of an appropriate policy file (see
-`etc/policy.v3cloudsample.json` as an example). This feature is deprecated and
-will be removed in a future release, in favor of strictly immutable domain IDs.
-"""))
-
 strict_password_check = cfg.BoolOpt(
     'strict_password_check',
     default=False,
@@ -202,7 +182,7 @@ will default to using the server's host name.
 
 notification_format = cfg.StrOpt(
     'notification_format',
-    default='basic',
+    default='cadf',
     choices=['basic', 'cadf'],
     help=utils.fmt("""
 Define the notification format for identity service events. A `basic`
@@ -215,14 +195,18 @@ auditing use cases.
 
 notification_opt_out = cfg.MultiStrOpt(
     'notification_opt_out',
-    default=[],
+    default=["identity.authenticate.success",
+             "identity.authenticate.pending",
+             "identity.authenticate.failed"],
     help=utils.fmt("""
-If left undefined, keystone will emit notifications for all types of events.
-You can reduce the number of notifications keystone emits by using this option
-to enumerate notification topics that should be suppressed. Values are expected
-to be in the form `identity.<resource_type>.<operation>`. This field can be set
-multiple times in order to opt-out of multiple notification topics. For
-example:
+You can reduce the number of notifications keystone emits by explicitly
+opting out. Keystone will not emit notifications that match the patterns
+expressed in this list. Values are expected to be in the form of
+`identity.<resource_type>.<operation>`. By default, all notifications
+related to authentication are automatically suppressed. This field can be
+set multiple times in order to opt-out of multiple notification topics. For
+example, the following suppresses notifications describing user creation or
+successful authentication events:
 notification_opt_out=identity.user.create
 notification_opt_out=identity.authenticate.success
 """))
@@ -240,7 +224,6 @@ ALL_OPTS = [
     member_role_name,
     crypt_strength,
     list_limit,
-    domain_id_immutable,
     strict_password_check,
     secure_proxy_ssl_header,
     insecure_debug,
