@@ -12,7 +12,7 @@
 
 """Main entry point into the Federation service."""
 
-from oslo_log import versionutils
+import uuid
 
 from keystone.common import cache
 from keystone.common import dependency
@@ -20,7 +20,6 @@ from keystone.common import extension
 from keystone.common import manager
 import keystone.conf
 from keystone import exception
-from keystone.federation.backends import base
 from keystone.federation import utils
 from keystone.i18n import _
 
@@ -31,7 +30,7 @@ MEMOIZE = cache.get_memoization_decorator(group='federation')
 CONF = keystone.conf.CONF
 EXTENSION_DATA = {
     'name': 'OpenStack Federation APIs',
-    'namespace': 'http://docs.openstack.org/identity/api/ext/'
+    'namespace': 'https://docs.openstack.org/identity/api/ext/'
                  'OS-FEDERATION/v1.0',
     'alias': 'OS-FEDERATION',
     'updated': '2013-12-17T12:00:0-00:00',
@@ -39,13 +38,14 @@ EXTENSION_DATA = {
     'links': [{
         'rel': 'describedby',
         'type': 'text/html',
-        'href': 'http://developer.openstack.org/api-ref-identity-v3-ext.html',
+        'href': 'https://developer.openstack.org/api-ref-identity-v3-ext.html',
     }]}
 extension.register_admin_extension(EXTENSION_DATA['alias'], EXTENSION_DATA)
 extension.register_public_extension(EXTENSION_DATA['alias'], EXTENSION_DATA)
 
 
 @dependency.provider('federation_api')
+@dependency.requires('resource_api')
 class Manager(manager.Manager):
     """Default pivot point for the Federation backend.
 
@@ -59,13 +59,28 @@ class Manager(manager.Manager):
     def __init__(self):
         super(Manager, self).__init__(CONF.federation.driver)
 
-        # Make sure it is a driver version we support, and if it is a legacy
-        # driver, then wrap it.
-        if isinstance(self.driver, base.FederationDriverV8):
-            self.driver = base.V9FederationWrapperForV8Driver(self.driver)
-        elif not isinstance(self.driver, base.FederationDriverV9):
-            raise exception.UnsupportedDriverVersion(
-                driver=CONF.federation.driver)
+    def create_idp(self, idp_id, idp):
+        if not idp.get('domain_id'):
+            idp['domain_id'] = self._create_idp_domain(idp_id)
+        else:
+            self._assert_valid_domain_id(idp['domain_id'])
+        return self.driver.create_idp(idp_id, idp)
+
+    def _create_idp_domain(self, idp_id):
+        domain_id = uuid.uuid4().hex
+        desc = 'Auto generated federated domain for Identity Provider: '
+        desc += idp_id
+        domain = {
+            'id': domain_id,
+            'name': domain_id,
+            'description': desc,
+            'enabled': True
+        }
+        self.resource_api.create_domain(domain['id'], domain)
+        return domain_id
+
+    def _assert_valid_domain_id(self, domain_id):
+        self.resource_api.get_domain(domain_id)
 
     @MEMOIZE
     def get_enabled_service_providers(self):
@@ -129,43 +144,3 @@ class Manager(manager.Manager):
         except exception.MappingNotFound:
             msg = _('Invalid mapping id: %s')
             raise exception.ValidationError(message=(msg % mapping_id))
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.federation.FederationDriverBase',
-    in_favor_of='keystone.federation.backends.base.FederationDriverBase',
-    remove_in=+1)
-class FederationDriverBase(base.FederationDriverBase):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.federation.FederationDriverV8',
-    in_favor_of='keystone.federation.backends.base.FederationDriverV8',
-    remove_in=+1)
-class FederationDriverV8(base.FederationDriverV8):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.federation.FederationDriverV9',
-    in_favor_of='keystone.federation.backends.base.FederationDriverV9',
-    remove_in=+1)
-class FederationDriverV9(base.FederationDriverV9):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.federation.V9FederationWrapperForV8Driver',
-    in_favor_of=(
-        'keystone.federation.backends.base.V9FederationWrapperForV8Driver'),
-    remove_in=+1)
-class V9FederationWrapperForV8Driver(base.V9FederationWrapperForV8Driver):
-    pass
-
-
-Driver = manager.create_legacy_driver(base.FederationDriverV8)
