@@ -36,6 +36,20 @@ class UserType(object):
     EPHEMERAL = 'ephemeral'
     LOCAL = 'local'
 
+ROLE_PROPERTIES = {
+    "type": "array",
+    "items": {
+        "type": "object",
+        "required": ["name"],
+        "properties": {
+            "name": {
+                "type": "string"
+            },
+            "additionalProperties": False
+        }
+    }
+}
+
 
 MAPPING_SCHEMA = {
     "type": "object",
@@ -72,6 +86,18 @@ MAPPING_SCHEMA = {
                                     },
                                     "additionalProperties": False
                                 },
+                                "projects": {
+                                    "type": "array",
+                                    "items": {
+                                        "type": "object",
+                                        "required": ["name", "roles"],
+                                        "additionalProperties": False,
+                                        "properties": {
+                                            "name": {"type": "string"},
+                                            "roles": ROLE_PROPERTIES
+                                        }
+                                    }
+                                },
                                 "group": {
                                     "type": "object",
                                     "oneOf": [
@@ -575,6 +601,7 @@ class RuleProcessor(object):
         group_ids = set()
         group_names = list()
         groups_by_domain = dict()
+        projects = []
 
         # if mapping yield no valid identity values, we should bail right away
         # instead of continuing on with a normalized bogus user
@@ -632,12 +659,15 @@ class RuleProcessor(object):
                         ast.literal_eval(identity_value['group_ids']))
                 except (ValueError, SyntaxError):
                     group_ids.update([identity_value['group_ids']])
+            if 'projects' in identity_value:
+                projects = identity_value['projects']
 
         normalize_user(user)
 
         return {'user': user,
                 'group_ids': list(group_ids),
-                'group_names': group_names}
+                'group_names': group_names,
+                'projects': projects}
 
     def _update_local_mapping(self, local, direct_maps):
         """Replace any {0}, {1} ... values with data from the assertion.
@@ -671,6 +701,9 @@ class RuleProcessor(object):
         for k, v in local.items():
             if isinstance(v, dict):
                 new_value = self._update_local_mapping(v, direct_maps)
+            elif isinstance(v, list):
+                new_value = [self._update_local_mapping(item, direct_maps)
+                             for item in v]
             else:
                 try:
                     new_value = v.format(*direct_maps)
