@@ -14,22 +14,25 @@
 
 import datetime
 
+from oslo_db import api as oslo_db_api
 import sqlalchemy
 
 from keystone.common import driver_hints
+from keystone.common import resource_options
 from keystone.common import sql
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 from keystone.identity.backends import base
+from keystone.identity.backends import resource_options as options
 from keystone.identity.backends import sql_model as model
 
 
 CONF = keystone.conf.CONF
 
 
-class Identity(base.IdentityDriverV8):
+class Identity(base.IdentityDriverBase):
     # NOTE(henry-nash): Override the __init__() method so as to take a
     # config parameter to enable sql to be used as a domain-specific driver.
     def __init__(self, conf=None):
@@ -56,6 +59,7 @@ class Identity(base.IdentityDriverV8):
         with sql.session_for_read() as session:
             try:
                 user_ref = self._get_user(session, user_id)
+                user_dict = base.filter_user(user_ref.to_dict())
             except exception.UserNotFound:
                 raise AssertionError(_('Invalid user / password'))
         if self._is_account_locked(user_id, user_ref):
@@ -70,7 +74,7 @@ class Identity(base.IdentityDriverV8):
         # successful auth, reset failed count if present
         if user_ref.local_user.failed_auth_count:
             self._reset_failed_auth(user_id)
-        return base.filter_user(user_ref.to_dict())
+        return user_dict
 
     def _is_account_locked(self, user_id, user_ref):
         """Check if the user account is locked.
@@ -83,6 +87,11 @@ class Identity(base.IdentityDriverV8):
         :returns Boolean: True if the account is locked; False otherwise
 
         """
+        ignore_option = user_ref.get_resource_option(
+            options.IGNORE_LOCKOUT_ATTEMPT_OPT.option_id)
+        if ignore_option and ignore_option.option_value is True:
+            return False
+
         attempts = user_ref.local_user.failed_auth_count or 0
         max_attempts = CONF.security_compliance.lockout_failure_attempts
         lockout_duration = CONF.security_compliance.lockout_duration
@@ -119,14 +128,45 @@ class Identity(base.IdentityDriverV8):
         user = utils.hash_user_password(user)
         with sql.session_for_write() as session:
             user_ref = model.User.from_dict(user)
+            if self._change_password_required(user_ref):
+                user_ref.password_ref.expires_at = datetime.datetime.utcnow()
             user_ref.created_at = datetime.datetime.utcnow()
             session.add(user_ref)
+            # Set resource options passed on creation
+            resource_options.resource_options_ref_to_mapper(
+                user_ref, model.UserOption)
             return base.filter_user(user_ref.to_dict())
 
+    def _change_password_required(self, user):
+        if not CONF.security_compliance.change_password_upon_first_use:
+            return False
+        ignore_option = user.get_resource_option(
+            options.IGNORE_CHANGE_PASSWORD_OPT.option_id)
+        return not (ignore_option and ignore_option.option_value is True)
+
+    def _create_password_expires_query(self, session, query, hints):
+        for filter_ in hints.filters:
+            if 'password_expires_at' == filter_['name']:
+                # Filter on users who's password expires based on the operator
+                # specified in `filter_['comparator']`
+                query = query.filter(sqlalchemy.and_(
+                    model.LocalUser.id == model.Password.local_user_id,
+                    filter_['comparator'](model.Password.expires_at,
+                                          filter_['value'])))
+        # Removes the `password_expired_at` filters so there are no errors
+        # if the call is filtered further. This is because the
+        # `password_expires_at` value is not stored in the `User` table but
+        # derived from the `Password` table's value `expires_at`.
+        hints.filters = [x for x in hints.filters if x['name'] !=
+                         'password_expires_at']
+        return query, hints
+
     @driver_hints.truncated
     def list_users(self, hints):
         with sql.session_for_read() as session:
             query = session.query(model.User).outerjoin(model.LocalUser)
+            query, hints = self._create_password_expires_query(session, query,
+                                                               hints)
             user_refs = sql.filter_limit_query(model.User, query, hints)
             return [base.filter_user(x.to_dict()) for x in user_refs]
 
@@ -165,6 +205,19 @@ class Identity(base.IdentityDriverV8):
             for attr in model.User.attributes:
                 if attr not in model.User.readonly_attributes:
                     setattr(user_ref, attr, getattr(new_user, attr))
+            # Move the "_resource_options" attribute over to the real user_ref
+            # so that resource_options.resource_options_ref_to_mapper can
+            # handle the work.
+            setattr(user_ref, '_resource_options',
+                    getattr(new_user, '_resource_options', {}))
+
+            # Move options into the proper attribute mapper construct
+            resource_options.resource_options_ref_to_mapper(
+                user_ref, model.UserOption)
+
+            if 'password' in user and self._change_password_required(user_ref):
+                user_ref.password_ref.expires_at = datetime.datetime.utcnow()
+
             user_ref.extra = new_user.extra
             return base.filter_user(
                 user_ref.to_dict(include_extra_dict=True))
@@ -178,11 +231,8 @@ class Identity(base.IdentityDriverV8):
         if unique_cnt > 1:
             for password_ref in user_ref.local_user.passwords:
                 if utils.check_password(password, password_ref.password):
-                    detail = _('The new password cannot be identical to a '
-                               'previous password. The number of previous '
-                               'passwords that must be unique is: '
-                               '%(unique_cnt)d') % {'unique_cnt': unique_cnt}
-                    raise exception.PasswordValidationError(detail=detail)
+                    raise exception.PasswordHistoryValidationError(
+                        unique_count=unique_cnt)
 
     def change_password(self, user_id, new_password):
         with sql.session_for_write() as session:
@@ -263,9 +313,12 @@ class Identity(base.IdentityDriverV8):
             query = query.join(model.UserGroupMembership)
             query = query.filter(
                 model.UserGroupMembership.group_id == group_id)
+            query, hints = self._create_password_expires_query(session, query,
+                                                               hints)
             query = sql.filter_limit_query(model.User, query, hints)
             return [base.filter_user(u.to_dict()) for u in query]
 
+    @oslo_db_api.wrap_db_retry(retry_on_deadlock=True)
     def delete_user(self, user_id):
         with sql.session_for_write() as session:
             ref = self._get_user(session, user_id)
