@@ -17,7 +17,7 @@ import mock
 from six.moves import range
 from testtools import matchers
 
-from keystone.catalog import core
+from keystone.catalog.backends import base
 from keystone.common import driver_hints
 from keystone import exception
 from keystone.tests import unit
@@ -187,7 +187,7 @@ class CatalogTests(object):
                           region_two['id'],
                           {'parent_region_id': region_four['id']})
 
-    @mock.patch.object(core.CatalogDriverV8,
+    @mock.patch.object(base.CatalogDriverBase,
                        "_ensure_no_circle_in_hierarchical_regions")
     def test_circular_regions_can_be_deleted(self, mock_ensure_on_circle):
         # turn off the enforcement so that cycles can be created for the test
@@ -560,8 +560,11 @@ class CatalogTests(object):
         enabled_endpoint_ref = self._create_endpoints()[1]
 
         user_id = uuid.uuid4().hex
-        project_id = uuid.uuid4().hex
-        catalog = self.catalog_api.get_v3_catalog(user_id, project_id)
+        # Use the project created by the default fixture since the project
+        # should exist if we want to filter the catalog by the project or
+        # replace the url with a valid project id.
+        catalog = self.catalog_api.get_v3_catalog(user_id,
+                                                  self.tenant_bar['id'])
 
         endpoint_ids = [x['id'] for x in catalog[0]['endpoints']]
         self.assertEqual([enabled_endpoint_ref['id']], endpoint_ids)
