@@ -16,11 +16,13 @@ import uuid
 import freezegun
 
 from keystone.common import controller
+from keystone.common import resource_options
 from keystone.common import sql
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
 from keystone.identity.backends import base
+from keystone.identity.backends import resource_options as iro
 from keystone.identity.backends import sql_model as model
 from keystone.tests.unit import test_backend_sql
 
@@ -28,6 +30,180 @@ from keystone.tests.unit import test_backend_sql
 CONF = keystone.conf.CONF
 
 
+class UserResourceOptionTests(test_backend_sql.SqlTests):
+    def setUp(self):
+        super(UserResourceOptionTests, self).setUp()
+        # RESET STATE OF REGISTRY OPTIONS
+        self.addCleanup(iro.register_user_options)
+        self.addCleanup(iro.USER_OPTIONS_REGISTRY._registered_options.clear)
+
+        self.option1 = resource_options.ResourceOption('opt1', 'option1')
+        self.option2 = resource_options.ResourceOption('opt2', 'option2')
+        self.cleanup_instance('option1', 'option2')
+
+        iro.USER_OPTIONS_REGISTRY._registered_options.clear()
+        iro.USER_OPTIONS_REGISTRY.register_option(self.option1)
+        iro.USER_OPTIONS_REGISTRY.register_option(self.option2)
+
+    def test_user_set_option_in_resource_option(self):
+        user = self._create_user(self._get_user_dict())
+        opt_value = uuid.uuid4().hex
+        user['options'][self.option1.option_name] = opt_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+        raw_ref = self._get_user_ref(user['id'])
+        self.assertIn(self.option1.option_id, raw_ref._resource_option_mapper)
+        self.assertEqual(
+            opt_value,
+            raw_ref._resource_option_mapper[
+                self.option1.option_id].option_value)
+        api_get_ref = self.identity_api.get_user(user['id'])
+        # Ensure options are properly set in a .get_user call.
+        self.assertEqual(opt_value,
+                         api_get_ref['options'][self.option1.option_name])
+
+    def test_user_add_update_delete_option_in_resource_option(self):
+        user = self._create_user(self._get_user_dict())
+
+        opt_value = uuid.uuid4().hex
+        new_opt_value = uuid.uuid4().hex
+
+        # Update user to add the new value option
+        user['options'][self.option1.option_name] = opt_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+
+        # Update the option Value and confirm it is updated
+        user['options'][self.option1.option_name] = new_opt_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(new_opt_value,
+                         new_ref['options'][self.option1.option_name])
+
+        # Set the option value to None, meaning delete the option
+        user['options'][self.option1.option_name] = None
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertNotIn(self.option1.option_name, new_ref['options'])
+
+    def test_user_add_delete_resource_option_existing_option_values(self):
+        user = self._create_user(self._get_user_dict())
+
+        opt_value = uuid.uuid4().hex
+        opt2_value = uuid.uuid4().hex
+
+        # Update user to add the new value option
+        user['options'][self.option1.option_name] = opt_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+
+        # Update the option value for option 2 and confirm it is updated and
+        # option1's value remains the same. Option 1 is not specified in the
+        # updated user ref.
+        del user['options'][self.option1.option_name]
+        user['options'][self.option2.option_name] = opt2_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+        self.assertEqual(opt2_value,
+                         new_ref['options'][self.option2.option_name])
+        raw_ref = self._get_user_ref(user['id'])
+        self.assertEqual(
+            opt_value,
+            raw_ref._resource_option_mapper[
+                self.option1.option_id].option_value)
+        self.assertEqual(
+            opt2_value,
+            raw_ref._resource_option_mapper[
+                self.option2.option_id].option_value)
+
+        # Set the option value to None, meaning delete the option, ensure
+        # option 2 still remains and has the right value
+        user['options'][self.option1.option_name] = None
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertNotIn(self.option1.option_name, new_ref['options'])
+        self.assertEqual(opt2_value,
+                         new_ref['options'][self.option2.option_name])
+        raw_ref = self._get_user_ref(user['id'])
+        self.assertNotIn(raw_ref._resource_option_mapper,
+                         self.option1.option_id)
+        self.assertEqual(
+            opt2_value,
+            raw_ref._resource_option_mapper[
+                self.option2.option_id].option_value)
+
+    def test_unregistered_resource_option_deleted(self):
+        user = self._create_user(self._get_user_dict())
+
+        opt_value = uuid.uuid4().hex
+        opt2_value = uuid.uuid4().hex
+
+        # Update user to add the new value option
+        user['options'][self.option1.option_name] = opt_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+
+        # Update the option value for option 2 and confirm it is updated and
+        # option1's value remains the same. Option 1 is not specified in the
+        # updated user ref.
+        del user['options'][self.option1.option_name]
+        user['options'][self.option2.option_name] = opt2_value
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+        self.assertEqual(opt2_value,
+                         new_ref['options'][self.option2.option_name])
+        raw_ref = self._get_user_ref(user['id'])
+        self.assertEqual(
+            opt_value,
+            raw_ref._resource_option_mapper[
+                self.option1.option_id].option_value)
+        self.assertEqual(
+            opt2_value,
+            raw_ref._resource_option_mapper[
+                self.option2.option_id].option_value)
+
+        # clear registered options and only re-register option1, update user
+        # and confirm option2 is gone from the ref and returned dict
+        iro.USER_OPTIONS_REGISTRY._registered_options.clear()
+        iro.USER_OPTIONS_REGISTRY.register_option(self.option1)
+        user['name'] = uuid.uuid4().hex
+        new_ref = self.identity_api.update_user(user['id'], user)
+        self.assertNotIn(self.option2.option_name, new_ref['options'])
+        self.assertEqual(opt_value,
+                         new_ref['options'][self.option1.option_name])
+        raw_ref = self._get_user_ref(user['id'])
+        self.assertNotIn(raw_ref._resource_option_mapper,
+                         self.option2.option_id)
+        self.assertEqual(
+            opt_value,
+            raw_ref._resource_option_mapper[
+                self.option1.option_id].option_value)
+
+    def _get_user_ref(self, user_id):
+        with sql.session_for_read() as session:
+            return session.query(model.User).get(user_id)
+
+    def _create_user(self, user_dict):
+        user_dict['id'] = uuid.uuid4().hex
+        user_dict = utils.hash_user_password(user_dict)
+        with sql.session_for_write() as session:
+            user_ref = model.User.from_dict(user_dict)
+            session.add(user_ref)
+            return base.filter_user(user_ref.to_dict())
+
+    def _get_user_dict(self):
+        user = {
+            'name': uuid.uuid4().hex,
+            'domain_id': CONF.identity.default_domain_id,
+            'enabled': True,
+            'password': uuid.uuid4().hex
+        }
+        return user
+
+
 class DisableInactiveUserTests(test_backend_sql.SqlTests):
     def setUp(self):
         super(DisableInactiveUserTests, self).setUp()
@@ -356,6 +532,25 @@ class LockingOutUserTests(test_backend_sql.SqlTests):
                           user_id=self.user['id'],
                           password=uuid.uuid4().hex)
 
+    def test_lock_out_for_ignored_user(self):
+        # mark the user as exempt from failed password attempts
+        # ignore user and reset password, password not expired
+        self.user['options'][iro.IGNORE_LOCKOUT_ATTEMPT_OPT.option_name] = True
+        self.identity_api.update_user(self.user['id'], self.user)
+
+        # fail authentication repeatedly the max number of times
+        self._fail_auth_repeatedly(self.user['id'])
+        # authenticate with wrong password, account should not be locked
+        self.assertRaises(AssertionError,
+                          self.identity_api.authenticate,
+                          self.make_request(),
+                          user_id=self.user['id'],
+                          password=uuid.uuid4().hex)
+        # authenticate with correct password, account should not be locked
+        self.identity_api.authenticate(self.make_request(),
+                                       user_id=self.user['id'],
+                                       password=self.password)
+
     def test_set_enabled_unlocks_user(self):
         # lockout user
         self._fail_auth_repeatedly(self.user['id'])
@@ -501,6 +696,33 @@ class PasswordExpiresValidationTests(test_backend_sql.SqlTests):
                                        user_id=user['id'],
                                        password=self.password)
 
+    def test_authenticate_with_expired_password_for_ignore_user_option(self):
+        # set user to have the 'ignore_password_expiry' option set to False
+        self.user_dict.setdefault('options', {})[
+            iro.IGNORE_PASSWORD_EXPIRY_OPT.option_name] = False
+        # set password created_at so that the password will expire
+        password_created_at = (
+            datetime.datetime.utcnow() -
+            datetime.timedelta(
+                days=CONF.security_compliance.password_expires_days + 1)
+        )
+        user = self._create_user(self.user_dict, password_created_at)
+        self.assertRaises(exception.PasswordExpired,
+                          self.identity_api.authenticate,
+                          self.make_request(),
+                          user_id=user['id'],
+                          password=self.password)
+
+        # update user to explicitly have the expiry option to True
+        user['options'][
+            iro.IGNORE_PASSWORD_EXPIRY_OPT.option_name] = True
+        user = self.identity_api.update_user(user['id'],
+                                             user)
+        # test password is not expired due to ignore option
+        self.identity_api.authenticate(self.make_request(),
+                                       user_id=user['id'],
+                                       password=self.password)
+
     def _get_test_user_dict(self, password):
         test_user_dict = {
             'id': uuid.uuid4().hex,
@@ -512,14 +734,16 @@ class PasswordExpiresValidationTests(test_backend_sql.SqlTests):
         return test_user_dict
 
     def _create_user(self, user_dict, password_created_at):
-        user_dict = utils.hash_user_password(user_dict)
+        # Bypass business logic and go straight for the identity driver
+        # (SQL in this case)
+        driver = self.identity_api.driver
+        driver.create_user(user_dict['id'], user_dict)
         with sql.session_for_write() as session:
-            user_ref = model.User.from_dict(user_dict)
+            user_ref = session.query(model.User).get(user_dict['id'])
             user_ref.password_ref.created_at = password_created_at
             user_ref.password_ref.expires_at = (
                 user_ref._get_password_expires_at(password_created_at))
-            session.add(user_ref)
-        return base.filter_user(user_ref.to_dict())
+            return base.filter_user(user_ref.to_dict())
 
 
 class MinimumPasswordAgeTests(test_backend_sql.SqlTests):
@@ -612,3 +836,97 @@ class MinimumPasswordAgeTests(test_backend_sql.SqlTests):
             for password_ref in user_ref.local_user.passwords:
                 password_ref.created_at = password_create_at - slightly_less
             latest_password.created_at = password_create_at
+
+
+class ChangePasswordRequiredAfterFirstUse(test_backend_sql.SqlTests):
+    def _create_user(self, password, change_password_upon_first_use):
+        self.config_fixture.config(
+            group='security_compliance',
+            change_password_upon_first_use=change_password_upon_first_use)
+        user_dict = {
+            'name': uuid.uuid4().hex,
+            'domain_id': CONF.identity.default_domain_id,
+            'enabled': True,
+            'password': password
+        }
+        return self.identity_api.create_user(user_dict)
+
+    def assertPasswordIsExpired(self, user_id, password):
+        self.assertRaises(exception.PasswordExpired,
+                          self.identity_api.authenticate,
+                          self.make_request(),
+                          user_id=user_id,
+                          password=password)
+
+    def assertPasswordIsNotExpired(self, user_id, password):
+        self.identity_api.authenticate(self.make_request(),
+                                       user_id=user_id,
+                                       password=password)
+
+    def test_password_expired_after_create(self):
+        # create user, password expired
+        initial_password = uuid.uuid4().hex
+        user = self._create_user(initial_password, True)
+        self.assertPasswordIsExpired(user['id'], initial_password)
+        # change password (self-service), password not expired
+        new_password = uuid.uuid4().hex
+        self.identity_api.change_password(self.make_request(),
+                                          user['id'],
+                                          initial_password,
+                                          new_password)
+        self.assertPasswordIsNotExpired(user['id'], new_password)
+
+    def test_password_expired_after_reset(self):
+        # create user with feature disabled, password not expired
+        initial_password = uuid.uuid4().hex
+        user = self._create_user(initial_password, False)
+        self.assertPasswordIsNotExpired(user['id'], initial_password)
+        # enable change_password_upon_first_use
+        self.config_fixture.config(
+            group='security_compliance',
+            change_password_upon_first_use=True)
+        # admin reset, password expired
+        admin_password = uuid.uuid4().hex
+        user['password'] = admin_password
+        self.identity_api.update_user(user['id'], user)
+        self.assertPasswordIsExpired(user['id'], admin_password)
+        # change password (self-service), password not expired
+        new_password = uuid.uuid4().hex
+        self.identity_api.change_password(self.make_request(),
+                                          user['id'],
+                                          admin_password,
+                                          new_password)
+        self.assertPasswordIsNotExpired(user['id'], new_password)
+
+    def test_password_not_expired_when_feature_disabled(self):
+        # create user with feature disabled
+        initial_password = uuid.uuid4().hex
+        user = self._create_user(initial_password, False)
+        self.assertPasswordIsNotExpired(user['id'], initial_password)
+        # admin reset
+        admin_password = uuid.uuid4().hex
+        user['password'] = admin_password
+        self.identity_api.update_user(user['id'], user)
+        self.assertPasswordIsNotExpired(user['id'], admin_password)
+
+    def test_password_not_expired_for_ignore_user(self):
+        # create user with feature disabled, password not expired
+        initial_password = uuid.uuid4().hex
+        user = self._create_user(initial_password, False)
+        self.assertPasswordIsNotExpired(user['id'], initial_password)
+        # enable change_password_upon_first_use
+        self.config_fixture.config(
+            group='security_compliance',
+            change_password_upon_first_use=True)
+        # ignore user and reset password, password not expired
+        user['options'][iro.IGNORE_CHANGE_PASSWORD_OPT.option_name] = True
+        admin_password = uuid.uuid4().hex
+        user['password'] = admin_password
+        self.identity_api.update_user(user['id'], user)
+        self.assertPasswordIsNotExpired(user['id'], admin_password)
+        # set ignore user to false and reset password, password is expired
+        user['options'][iro.IGNORE_CHANGE_PASSWORD_OPT.option_name] = False
+        admin_password = uuid.uuid4().hex
+        user['password'] = admin_password
+        self.identity_api.update_user(user['id'], user)
+        self.assertPasswordIsExpired(user['id'], admin_password)
