@@ -1626,3 +1626,91 @@ MAPPING_UNICODE = {
         },
     ],
 }
+
+MAPPING_PROJECTS = {
+    "rules": [
+        {
+            "local": [
+                {
+                    "user": {
+                        "name": "{0}"
+                    }
+                },
+                {
+                    "projects": [
+                        {"name": "Production",
+                         "roles": [{"name": "observer"}]},
+                        {"name": "Staging",
+                         "roles": [{"name": "member"}]},
+                        {"name": "Project for {0}",
+                         "roles": [{"name": "admin"}]},
+                    ],
+                }
+            ],
+            "remote": [
+                {
+                    "type": "UserName"
+                },
+                {
+                    "type": "Email",
+                },
+                {
+                    "type": "orgPersonType",
+                    "any_one_of": [
+                        "Employee"
+                    ]
+                }
+            ]
+        }
+    ]
+}
+
+MAPPING_PROJECTS_WITHOUT_ROLES = {
+    "rules": [
+        {
+            "local": [
+                {
+                    "user": {
+                        "name": "{0}"
+                    },
+                    "projects": [
+                        {"name": "a"},
+                        {"name": "b"},
+                        {"name": "Project for {0}"},
+                    ],
+                }
+            ],
+            "remote": [
+                {
+                    "type": "UserName"
+                }
+            ]
+        },
+    ]
+}
+
+MAPPING_PROJECTS_WITHOUT_NAME = {
+    "rules": [
+        {
+            "local": [
+                {
+                    "user": {
+                        "name": "{0}"
+                    },
+                    "projects": [
+                        {"roles": [{"name": "observer"}]},
+                        {"name": "Staging",
+                         "roles": [{"name": "member"}]},
+                        {"name": "Project for {0}",
+                         "roles": [{"name": "admin"}]},
+                    ]
+                }
+            ],
+            "remote": [
+                {
+                    "type": "UserName"
+                }
+            ]
+        },
+    ]
+}
