@@ -18,16 +18,23 @@ import uuid
 import fixtures
 import freezegun
 import mock
+from oslo_db import exception as oslo_db_exception
 from oslo_log import log
 from six.moves import http_client
 from testtools import matchers
 
 from keystone.common import controller
+from keystone.common import sql
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
+from keystone.identity.backends import base as identity_base
+from keystone.identity.backends import resource_options as options
+from keystone.identity.backends import sql_model as model
 from keystone.tests import unit
 from keystone.tests.unit import ksfixtures
+from keystone.tests.unit.ksfixtures import database
+from keystone.tests.unit import mapping_fixtures
 from keystone.tests.unit import test_v3
 
 
@@ -38,7 +45,6 @@ CONF = keystone.conf.CONF
 # removed. This was moved to it's own testcase so it can setup the
 # admin_token_auth pipeline without impacting other tests.
 class IdentityTestCaseStaticAdminToken(test_v3.RestfulTestCase):
-    EXTENSION_TO_ADD = 'admin_token_auth'
 
     def config_overrides(self):
         super(IdentityTestCaseStaticAdminToken, self).config_overrides()
@@ -486,20 +492,18 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.v3_create_token(new_password_auth)
 
     def test_update_user_domain_id(self):
-        """Call ``PATCH /users/{user_id}`` with domain_id."""
+        """Call ``PATCH /users/{user_id}`` with domain_id.
+
+        A user's `domain_id` is immutable. Ensure that any attempts to update
+        the `domain_id` of a user fails.
+        """
         user = unit.new_user_ref(domain_id=self.domain['id'])
         user = self.identity_api.create_user(user)
         user['domain_id'] = CONF.identity.default_domain_id
-        r = self.patch('/users/%(user_id)s' % {
+        self.patch('/users/%(user_id)s' % {
             'user_id': user['id']},
             body={'user': user},
             expected_status=exception.ValidationError.code)
-        self.config_fixture.config(domain_id_immutable=False)
-        user['domain_id'] = self.domain['id']
-        r = self.patch('/users/%(user_id)s' % {
-            'user_id': user['id']},
-            body={'user': user})
-        self.assertValidUserResponse(r, user)
 
     def test_delete_user(self):
         """Call ``DELETE /users/{user_id}``.
@@ -551,6 +555,43 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         r = self.credential_api.get_credential(credential2['id'])
         self.assertDictEqual(credential2, r)
 
+    def test_delete_user_retries_on_deadlock(self):
+        patcher = mock.patch('sqlalchemy.orm.query.Query.delete',
+                             autospec=True)
+
+        class FakeDeadlock(object):
+            def __init__(self, mock_patcher):
+                self.deadlock_count = 2
+                self.mock_patcher = mock_patcher
+                self.patched = True
+
+            def __call__(self, *args, **kwargs):
+                if self.deadlock_count > 1:
+                    self.deadlock_count -= 1
+                else:
+                    self.mock_patcher.stop()
+                    self.patched = False
+                raise oslo_db_exception.DBDeadlock
+
+        sql_delete_mock = patcher.start()
+        side_effect = FakeDeadlock(patcher)
+        sql_delete_mock.side_effect = side_effect
+
+        user_ref = unit.create_user(self.identity_api,
+                                    domain_id=self.domain['id'])
+
+        try:
+            self.identity_api.delete_user(user_id=user_ref['id'])
+        finally:
+            if side_effect.patched:
+                patcher.stop()
+
+        call_count = sql_delete_mock.call_count
+
+        # initial attempt + 1 retry
+        delete_user_attempt_count = 2
+        self.assertEqual(call_count, delete_user_attempt_count)
+
     # group crud tests
 
     def test_create_group(self):
@@ -593,18 +634,16 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.assertValidGroupResponse(r, group)
 
     def test_update_group_domain_id(self):
-        """Call ``PATCH /groups/{group_id}`` with domain_id."""
+        """Call ``PATCH /groups/{group_id}`` with domain_id.
+
+        A group's `domain_id` is immutable. Ensure that any attempts to update
+        the `domain_id` of a group fails.
+        """
         self.group['domain_id'] = CONF.identity.default_domain_id
-        r = self.patch('/groups/%(group_id)s' % {
+        self.patch('/groups/%(group_id)s' % {
             'group_id': self.group['id']},
             body={'group': self.group},
             expected_status=exception.ValidationError.code)
-        self.config_fixture.config(domain_id_immutable=False)
-        self.group['domain_id'] = self.domain['id']
-        r = self.patch('/groups/%(group_id)s' % {
-            'group_id': self.group['id']},
-            body={'group': self.group})
-        self.assertValidGroupResponse(r, self.group)
 
     def test_delete_group(self):
         """Call ``DELETE /groups/{group_id}``."""
@@ -772,10 +811,10 @@ class IdentityV3toV2MethodsTestCase(unit.TestCase):
         self.assertDictEqual(self.expected_user_no_tenant_id, self.user4)
 
 
-class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
+class ChangePasswordTestCase(test_v3.RestfulTestCase):
 
     def setUp(self):
-        super(UserSelfServiceChangingPasswordsTestCase, self).setUp()
+        super(ChangePasswordTestCase, self).setUp()
         self.user_ref = unit.create_user(self.identity_api,
                                          domain_id=self.domain['id'])
         self.token = self.get_request_token(self.user_ref['password'],
@@ -796,6 +835,26 @@ class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
                          token=self.token,
                          expected_status=expected_status)
 
+
+class UserSelfServiceChangingPasswordsTestCase(ChangePasswordTestCase):
+
+    def setUp(self):
+        super(UserSelfServiceChangingPasswordsTestCase, self).setUp()
+
+    def _create_user_with_expired_password(self):
+        expire_days = CONF.security_compliance.password_expires_days + 1
+        time = (
+            datetime.datetime.utcnow() -
+            datetime.timedelta(expire_days)
+        )
+        password = uuid.uuid4().hex
+        user_ref = unit.new_user_ref(domain_id=self.domain_id,
+                                     password=password)
+        with freezegun.freeze_time(time):
+            self.user_ref = self.identity_api.create_user(user_ref)
+
+        return password
+
     def test_changing_password(self):
         # original password works
         token_id = self.get_request_token(self.user_ref['password'],
@@ -892,9 +951,117 @@ class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
         self.assertNotIn(self.user_ref['password'], log_fix.output)
         self.assertNotIn(new_password, log_fix.output)
 
+    def test_changing_expired_password_succeeds(self):
+        self.config_fixture.config(group='security_compliance',
+                                   password_expires_days=2)
+        password = self._create_user_with_expired_password()
 
-class PasswordValidationTestCase(UserSelfServiceChangingPasswordsTestCase):
-    """Test password validation."""
+        new_password = uuid.uuid4().hex
+        self.change_password(password=new_password,
+                             original_password=password,
+                             expected_status=http_client.NO_CONTENT)
+        # new password works
+        self.get_request_token(new_password,
+                               expected_status=http_client.CREATED)
+
+    def test_changing_expired_password_with_disabled_user_fails(self):
+        self.config_fixture.config(group='security_compliance',
+                                   password_expires_days=2)
+
+        password = self._create_user_with_expired_password()
+        # disable the user account
+        self.user_ref['enabled'] = False
+        self.patch('/users/%s' % self.user_ref['id'],
+                   body={'user': self.user_ref})
+
+        new_password = uuid.uuid4().hex
+        self.change_password(password=new_password,
+                             original_password=password,
+                             expected_status=http_client.UNAUTHORIZED)
+
+    def test_change_password_required_upon_first_use_for_create(self):
+        self.config_fixture.config(group='security_compliance',
+                                   change_password_upon_first_use=True)
+
+        # create user
+        self.user_ref = unit.create_user(self.identity_api,
+                                         domain_id=self.domain['id'])
+
+        # attempt to authenticate with create user password
+        self.get_request_token(self.user_ref['password'],
+                               expected_status=http_client.UNAUTHORIZED)
+
+        # self-service change password
+        new_password = uuid.uuid4().hex
+        self.change_password(password=new_password,
+                             original_password=self.user_ref['password'],
+                             expected_status=http_client.NO_CONTENT)
+
+        # authenticate with the new password
+        self.token = self.get_request_token(new_password, http_client.CREATED)
+
+    def test_change_password_required_upon_first_use_for_admin_reset(self):
+        self.config_fixture.config(group='security_compliance',
+                                   change_password_upon_first_use=True)
+
+        # admin reset
+        reset_password = uuid.uuid4().hex
+        user_password = {'password': reset_password}
+        self.identity_api.update_user(self.user_ref['id'], user_password)
+
+        # attempt to authenticate with admin reset password
+        self.get_request_token(reset_password,
+                               expected_status=http_client.UNAUTHORIZED)
+
+        # self-service change password
+        new_password = uuid.uuid4().hex
+        self.change_password(password=new_password,
+                             original_password=reset_password,
+                             expected_status=http_client.NO_CONTENT)
+
+        # authenticate with the new password
+        self.token = self.get_request_token(new_password, http_client.CREATED)
+
+    def test_change_password_required_upon_first_use_ignore_user(self):
+        self.config_fixture.config(group='security_compliance',
+                                   change_password_upon_first_use=True)
+
+        # ignore user and reset password
+        reset_password = uuid.uuid4().hex
+        self.user_ref['password'] = reset_password
+        ignore_opt_name = options.IGNORE_CHANGE_PASSWORD_OPT.option_name
+        self.user_ref['options'][ignore_opt_name] = True
+        self.identity_api.update_user(self.user_ref['id'], self.user_ref)
+
+        # authenticate with the reset password
+        self.token = self.get_request_token(reset_password,
+                                            http_client.CREATED)
+
+    def test_lockout_exempt(self):
+        self.config_fixture.config(group='security_compliance',
+                                   lockout_failure_attempts=1)
+
+        # create user
+        self.user_ref = unit.create_user(self.identity_api,
+                                         domain_id=self.domain['id'])
+
+        # update the user, mark her as exempt from lockout
+        ignore_opt_name = options.IGNORE_LOCKOUT_ATTEMPT_OPT.option_name
+        self.user_ref['options'][ignore_opt_name] = True
+        self.identity_api.update_user(self.user_ref['id'], self.user_ref)
+
+        # fail to auth, this should lockout the user, since we're allowed
+        # one failure, but we're exempt from lockout!
+        bad_password = uuid.uuid4().hex
+        self.token = self.get_request_token(bad_password,
+                                            http_client.UNAUTHORIZED)
+
+        # attempt to authenticate with correct password
+        self.get_request_token(self.user_ref['password'],
+                               expected_status=http_client.CREATED)
+
+
+class PasswordValidationTestCase(ChangePasswordTestCase):
 
     def setUp(self):
         super(PasswordValidationTestCase, self).setUp()
@@ -946,3 +1113,125 @@ class PasswordValidationTestCase(UserSelfServiceChangingPasswordsTestCase):
         self.change_password(password='mypas2',
                              original_password=self.user_ref['password'],
                              expected_status=http_client.BAD_REQUEST)
+
+
+class UserFederatedAttributesTests(test_v3.RestfulTestCase):
+    def _create_federated_attributes(self):
+        # Create the idp
+        idp = {
+            'id': uuid.uuid4().hex,
+            'enabled': True,
+            'description': uuid.uuid4().hex
+        }
+        self.federation_api.create_idp(idp['id'], idp)
+        # Create the mapping
+        mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
+        mapping['id'] = uuid.uuid4().hex
+        self.federation_api.create_mapping(mapping['id'], mapping)
+        # Create the protocol
+        protocol = {
+            'id': uuid.uuid4().hex,
+            'mapping_id': mapping['id']
+        }
+        self.federation_api.create_protocol(idp['id'],
+                                            protocol['id'],
+                                            protocol)
+        return idp, protocol
+
+    def _create_user_with_federated_user(self, user, fed_dict):
+        with sql.session_for_write() as session:
+            federated_ref = model.FederatedUser.from_dict(fed_dict)
+            user_ref = model.User.from_dict(user)
+            user_ref.created_at = datetime.datetime.utcnow()
+            user_ref.federated_users.append(federated_ref)
+            session.add(user_ref)
+            return identity_base.filter_user(user_ref.to_dict())
+
+    def setUp(self):
+        super(UserFederatedAttributesTests, self).setUp()
+        self.useFixture(database.Database())
+        self.load_backends()
+        # Create the federated object
+        idp, protocol = self._create_federated_attributes()
+        self.fed_dict = unit.new_federated_user_ref()
+        self.fed_dict['idp_id'] = idp['id']
+        self.fed_dict['protocol_id'] = protocol['id']
+        self.fed_dict['unique_id'] = "jdoe"
+        # Create the domain_id, user, and federated_user relationship
+        self.domain = unit.new_domain_ref()
+        self.resource_api.create_domain(self.domain['id'], self.domain)
+        self.fed_user = unit.new_user_ref(domain_id=self.domain['id'])
+        self.fed_user = self._create_user_with_federated_user(self.fed_user,
+                                                              self.fed_dict)
+        # Create two new fed_users which will have the same idp and protocol
+        # but be completely different from the first fed_user
+        # Create a new idp and protocol for fed_user2 and 3
+        idp, protocol = self._create_federated_attributes()
+        self.fed_dict2 = unit.new_federated_user_ref()
+        self.fed_dict2['idp_id'] = idp['id']
+        self.fed_dict2['protocol_id'] = protocol['id']
+        self.fed_dict2['unique_id'] = "ravelar"
+        self.fed_user2 = unit.new_user_ref(domain_id=self.domain['id'])
+        self.fed_user2 = self._create_user_with_federated_user(self.fed_user2,
+                                                               self.fed_dict2)
+        self.fed_dict3 = unit.new_federated_user_ref()
+        self.fed_dict3['idp_id'] = idp['id']
+        self.fed_dict3['protocol_id'] = protocol['id']
+        self.fed_dict3['unique_id'] = "jsmith"
+        self.fed_user3 = unit.new_user_ref(domain_id=self.domain['id'])
+        self.fed_user3 = self._create_user_with_federated_user(self.fed_user3,
+                                                               self.fed_dict3)
+
+    def _test_list_users_with_federated_parameter(self, parameter):
+        # construct the resource url based off what's passed in parameter
+        resource_url = ('/users?%s=%s'
+                        % (parameter[0], self.fed_dict[parameter[0]]))
+        for attr in parameter[1:]:
+            resource_url += '&%s=%s' % (attr, self.fed_dict[attr])
+        r = self.get(resource_url)
+        # Check that only one out of 3 fed_users is matched by calling the api
+        # and that it is a valid response
+        self.assertEqual(1, len(r.result['users']))
+        self.assertValidUserListResponse(r, ref=self.fed_user,
+                                         resource_url=resource_url)
+        # Since unique_id will always return one user if matching for unique_id
+        # in the query, we rule out unique_id for the next tests
+        if not any('unique_id' in x for x in parameter):
+            # Check that we get two matches here since fed_user2 and fed_user3
+            # both have the same idp and protocol
+            resource_url = ('/users?%s=%s'
+                            % (parameter[0], self.fed_dict2[parameter[0]]))
+            for attr in parameter[1:]:
+                resource_url += '&%s=%s' % (attr, self.fed_dict2[attr])
+            r = self.get(resource_url)
+            self.assertEqual(2, len(r.result['users']))
+            self.assertValidUserListResponse(r, ref=self.fed_user2,
+                                             resource_url=resource_url)
+
+    def test_list_users_with_idp_id(self):
+        attribute = ['idp_id']
+        self._test_list_users_with_federated_parameter(attribute)
+
+    def test_list_users_with_protocol_id(self):
+        attribute = ['protocol_id']
+        self._test_list_users_with_federated_parameter(attribute)
+
+    def test_list_users_with_unique_id(self):
+        attribute = ['unique_id']
+        self._test_list_users_with_federated_parameter(attribute)
+
+    def test_list_users_with_idp_id_and_unique_id(self):
+        attribute = ['idp_id', 'unique_id']
+        self._test_list_users_with_federated_parameter(attribute)
+
+    def test_list_users_with_idp_id_and_protocol_id(self):
+        attribute = ['idp_id', 'protocol_id']
+        self._test_list_users_with_federated_parameter(attribute)
+
+    def test_list_users_with_protocol_id_and_unique_id(self):
+        attribute = ['protocol_id', 'unique_id']
+        self._test_list_users_with_federated_parameter(attribute)
+
+    def test_list_users_with_all_federated_attributes(self):
+        attribute = ['idp_id', 'protocol_id', 'unique_id']
+        self._test_list_users_with_federated_parameter(attribute)
