@@ -17,20 +17,18 @@
 import abc
 import copy
 
-from oslo_config import cfg
 from oslo_log import log
-from oslo_utils import timeutils
 import six
 
 from keystone.common import cache
 from keystone.common import dependency
 from keystone.common import manager
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _LW
-from keystone.token import utils
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 MEMOIZE = cache.get_memoization_decorator(group='token')
 REVOCATION_MEMOIZE = cache.get_memoization_decorator(group='token',
@@ -38,7 +36,7 @@ REVOCATION_MEMOIZE = cache.get_memoization_decorator(group='token',
 
 
 @dependency.requires('assignment_api', 'identity_api', 'resource_api',
-                     'token_provider_api', 'trust_api')
+                     'trust_api')
 class PersistenceManager(manager.Manager):
     """Default pivot point for the Token Persistence backend.
 
@@ -52,22 +50,8 @@ class PersistenceManager(manager.Manager):
     def __init__(self):
         super(PersistenceManager, self).__init__(CONF.token.driver)
 
-    def _assert_valid(self, token_id, token_ref):
-        """Raise TokenNotFound if the token is expired."""
-        current_time = timeutils.normalize_time(timeutils.utcnow())
-        expires = token_ref.get('expires')
-        if not expires or current_time > timeutils.normalize_time(expires):
-            raise exception.TokenNotFound(token_id=token_id)
-
     def get_token(self, token_id):
-        unique_id = utils.generate_unique_id(token_id)
-        token_ref = self._get_token(unique_id)
-        # NOTE(morganfainberg): Lift expired checking to the manager, there is
-        # no reason to make the drivers implement this check. With caching,
-        # self._get_token could return an expired token. Make sure we behave
-        # as expected and raise TokenNotFound on those instances.
-        self._assert_valid(token_id, token_ref)
-        return token_ref
+        return self._get_token(token_id)
 
     @MEMOIZE
     def _get_token(self, token_id):
@@ -75,23 +59,21 @@ class PersistenceManager(manager.Manager):
         return self.driver.get_token(token_id)
 
     def create_token(self, token_id, data):
-        unique_id = utils.generate_unique_id(token_id)
         data_copy = copy.deepcopy(data)
-        data_copy['id'] = unique_id
-        ret = self.driver.create_token(unique_id, data_copy)
+        data_copy['id'] = token_id
+        ret = self.driver.create_token(token_id, data_copy)
         if MEMOIZE.should_cache(ret):
             # NOTE(morganfainberg): when doing a cache set, you must pass the
             # same arguments through, the same as invalidate (this includes
             # "self"). First argument is always the value to be cached
-            self._get_token.set(ret, self, unique_id)
+            self._get_token.set(ret, self, token_id)
         return ret
 
     def delete_token(self, token_id):
         if not CONF.token.revoke_by_id:
             return
-        unique_id = utils.generate_unique_id(token_id)
-        self.driver.delete_token(unique_id)
-        self._invalidate_individual_token_cache(unique_id)
+        self.driver.delete_token(token_id)
+        self._invalidate_individual_token_cache(token_id)
         self.invalidate_revocation_list()
 
     def delete_tokens(self, user_id, tenant_id=None, trust_id=None,
@@ -101,8 +83,7 @@ class PersistenceManager(manager.Manager):
         token_list = self.driver.delete_tokens(user_id, tenant_id, trust_id,
                                                consumer_id)
         for token_id in token_list:
-            unique_id = utils.generate_unique_id(token_id)
-            self._invalidate_individual_token_cache(unique_id)
+            self._invalidate_individual_token_cache(token_id)
         self.invalidate_revocation_list()
 
     @REVOCATION_MEMOIZE
@@ -184,13 +165,7 @@ class PersistenceManager(manager.Manager):
         # NOTE(morganfainberg): invalidate takes the exact same arguments as
         # the normal method, this means we need to pass "self" in (which gets
         # stripped off).
-
-        # FIXME(morganfainberg): Does this cache actually need to be
-        # invalidated? We maintain a cached revocation list, which should be
-        # consulted before accepting a token as valid.  For now we will
-        # do the explicit individual token invalidation.
         self._get_token.invalidate(self, token_id)
-        self.token_provider_api.invalidate_individual_token_cache(token_id)
 
 
 @dependency.requires('token_provider_api')
@@ -225,7 +200,7 @@ class Manager(object):
 
 
 @six.add_metaclass(abc.ABCMeta)
-class TokenDriverV8(object):
+class TokenDriverBase(object):
     """Interface description for a Token driver."""
 
     @abc.abstractmethod
@@ -255,7 +230,6 @@ class TokenDriverV8(object):
                 id=token_id,
                 user=user_ref,
                 tenant=tenant_ref,
-                metadata=metadata_ref
             }
 
         :type data: dict
@@ -266,7 +240,7 @@ class TokenDriverV8(object):
 
     @abc.abstractmethod
     def delete_token(self, token_id):
-        """Deletes a token by id.
+        """Delete a token by id.
 
         :param token_id: identity of the token
         :type token_id: string
@@ -279,7 +253,7 @@ class TokenDriverV8(object):
     @abc.abstractmethod
     def delete_tokens(self, user_id, tenant_id=None, trust_id=None,
                       consumer_id=None):
-        """Deletes tokens by user.
+        """Delete tokens by user.
 
         If the tenant_id is not None, only delete the tokens by user id under
         the specified tenant.
@@ -320,7 +294,7 @@ class TokenDriverV8(object):
     @abc.abstractmethod
     def _list_tokens(self, user_id, tenant_id=None, trust_id=None,
                      consumer_id=None):
-        """Returns a list of current token_id's for a user
+        """Return a list of current token_id's for a user.
 
         This is effectively a private method only used by the ``delete_tokens``
         method and should not be called by anything outside of the
@@ -341,7 +315,7 @@ class TokenDriverV8(object):
 
     @abc.abstractmethod
     def list_revoked_tokens(self):
-        """Returns a list of all revoked tokens
+        """Return a list of all revoked tokens.
 
         :returns: list of token_id's
 
@@ -352,6 +326,3 @@ class TokenDriverV8(object):
     def flush_expired_tokens(self):
         """Archive or delete tokens that have expired."""
         raise exception.NotImplemented()  # pragma: no cover
-
-
-Driver = manager.create_legacy_driver(TokenDriverV8)
