@@ -10,16 +10,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo_config import cfg
+import os
+
 
 from keystone.common import dependency
 from keystone.common import utils as ks_utils
+import keystone.conf
 from keystone.federation import constants as federation_constants
+from keystone.i18n import _
 from keystone.token.providers import common
 from keystone.token.providers.fernet import token_formatters as tf
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 @dependency.requires('trust_api', 'oauth_api', 'identity_api')
@@ -27,20 +30,28 @@ class Provider(common.BaseProvider):
     def __init__(self, *args, **kwargs):
         super(Provider, self).__init__(*args, **kwargs)
 
+        # NOTE(lbragstad): We add these checks here because if the fernet
+        # provider is going to be used and either the `key_repository` is empty
+        # or doesn't exist we should fail, hard. It doesn't make sense to start
+        # keystone and just 500 because we can't do anything with an empty or
+        # non-existant key repository.
+        if not os.path.exists(CONF.fernet_tokens.key_repository):
+            subs = {'key_repo': CONF.fernet_tokens.key_repository}
+            raise SystemExit(_('%(key_repo)s does not exist') % subs)
+        if not os.listdir(CONF.fernet_tokens.key_repository):
+            subs = {'key_repo': CONF.fernet_tokens.key_repository}
+            raise SystemExit(_('%(key_repo)s does not contain keys, use '
+                               'keystone-manage fernet_setup to create '
+                               'Fernet keys.') % subs)
+
         self.token_formatter = tf.TokenFormatter()
 
     def needs_persistence(self):
         """Should the token be written to a backend."""
         return False
 
-    def issue_v2_token(self, *args, **kwargs):
-        token_id, token_data = super(Provider, self).issue_v2_token(
-            *args, **kwargs)
-        self._build_issued_at_info(token_id, token_data)
-        return token_id, token_data
-
-    def issue_v3_token(self, *args, **kwargs):
-        token_id, token_data = super(Provider, self).issue_v3_token(
+    def issue_token(self, *args, **kwargs):
+        token_id, token_data = super(Provider, self).issue_token(
             *args, **kwargs)
         self._build_issued_at_info(token_id, token_data)
         return token_id, token_data
@@ -139,7 +150,7 @@ class Provider(common.BaseProvider):
 
         """
         group_ids = [x['id'] for x in federated_dict['group_ids']]
-        self.v3_token_data_helper.populate_roles_for_groups(
+        self.v3_token_data_helper.populate_roles_for_federated_user(
             token_dict, group_ids, project_id, domain_id, user_id)
 
     def _extract_v2_token_data(self, token_data):
@@ -151,7 +162,7 @@ class Provider(common.BaseProvider):
             methods.append('token')
         project_id = token_data['access']['token'].get('tenant', {}).get('id')
         domain_id = None
-        trust_id = None
+        trust_id = token_data['access'].get('trust', {}).get('id')
         access_token_id = None
         federated_info = None
         return (user_id, expires_at, audit_ids, methods, domain_id, project_id,
